@@ -8,6 +8,7 @@ import locale
 import tempfile
 import subprocess
 import codecs
+import mintcommon
 
 try:
     import _thread as thread
@@ -63,6 +64,7 @@ class IMLanguage():
         self.app = app
         self.packages = []
         self.missing_packages = []
+        self.apt = mintcommon.APT(self.app.window)
 
         self.label = Gtk.Label()
         self.label.set_markup(name)
@@ -103,16 +105,24 @@ class IMLanguage():
                         self.packages.append(line)
 
     def install(self, widget):
-        xid = str(self.app.window.get_window().get_xid())
         if len(self.missing_packages) > 0:
-            refresh = "no"
-            if not self.app.cache_updated:
-                refresh = "yes"
-                self.app.cache_updated = True
-            cmd = ["gksu", "/usr/lib/linuxmint/mintlocale/synaptic-install-packages", refresh, xid]
-            cmd = cmd + self.missing_packages
-            comnd = subprocess.Popen(cmd)
-            returnCode = comnd.wait()
+            self.app.lock_input_methods()
+            if self.app.cache_updated:
+                self.apt.set_finished_callback(self.on_install_finished)
+                self.apt.set_cancelled_callback(self.on_install_finished)
+                self.apt.install_packages(self.missing_packages)
+            else:
+                self.apt.set_finished_callback(self.on_update_finished)
+                self.apt.update_cache()
+
+    def on_update_finished(self, transaction=None, exit_state=None):
+        self.app.cache_updated = True
+        self.apt.set_finished_callback(self.on_install_finished)
+        self.apt.set_cancelled_callback(self.on_install_finished)
+        self.apt.install_packages(self.missing_packages)
+
+    def on_install_finished(self, transaction=None, exit_state=None):
+        print("Finished")
         self.app.check_input_methods()
 
     def update_status(self, cache):
@@ -122,6 +132,7 @@ class IMLanguage():
                 self.missing_packages.append(package)
         if len(self.missing_packages) > 0:
             self.button.show()
+            self.button.set_sensitive(True)
             self.button.set_tooltip_text("\n".join(self.missing_packages))
         else:
             self.settings_row.show_alternative_widget()
@@ -333,18 +344,6 @@ class SettingsBox(Gtk.Frame):
         toolbar.add(title_holder)
         self.box.add(toolbar)
 
-        toolbar_separator = Gtk.Separator(orientation=Gtk.Orientation.HORIZONTAL)
-        self.box.add(toolbar_separator)
-        separator_context = toolbar_separator.get_style_context()
-        frame_color = frame_style.get_border_color(Gtk.StateFlags.NORMAL).to_string()
-        css_provider = Gtk.CssProvider()
-        css_vals = ".separator { -GtkWidget-wide-separators: 0;    \
-                                                      color: %s;   \
-                                                }" % frame_color
-
-        css_provider.load_from_data(css_vals.encode('utf-8'))
-        separator_context.add_provider(css_provider, Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION)
-
         self.list_box = Gtk.ListBox()
         self.list_box.set_selection_mode(Gtk.SelectionMode.NONE)
         self.list_box.set_header_func(list_header_func, None)
@@ -422,10 +421,10 @@ class MintLocale:
 
         self.window = self.builder.get_object("main_window")
 
-        self.builder.get_object("main_window").connect("destroy", Gtk.main_quit)
+        self.window.connect("destroy", Gtk.main_quit)
 
         # set up larger components.
-        self.builder.get_object("main_window").set_title(_("Language Settings"))
+        self.window.set_title(_("Language Settings"))
 
         self.toolbar = Gtk.Toolbar()
         self.toolbar.get_style_context().add_class("primary-toolbar")
@@ -610,24 +609,35 @@ class MintLocale:
         if (show_input_methods):
             page.show()
             stack.set_visible_child(page)
+            self.window.set_icon_name("mintlocale-im")
+        else:
+            self.window.set_icon_name("preferences-desktop-locale")
 
     def button_system_language_clicked(self, button):
         print("Setting system locale: language '%s', region '%s'" % (self.current_language, self.current_region))
-        subprocess.call(['gksu', 'set-default-locale', self.locale_path, self.current_language, self.current_region])
+        subprocess.call(['pkexec', 'set-default-locale', self.locale_path, self.current_language, self.current_region])
         self.set_system_locale()
         pass
 
     def button_install_remove_clicked(self, button):
-        os.system("gksu add-remove-locales")
+        os.system("pkexec add-remove-locales")
         self.build_lang_list()
         self.set_system_locale()
         self.set_num_installed()
 
+    def lock_input_methods(self):
+        # lock all buttons while we install packages
+        for im in self.im_languages:
+            im.button.set_sensitive(False)
+        self.im_combo.set_sensitive(False)
+
     def check_input_methods(self):
         if not self.ImConfig.available():
             self.im_combo.set_sensitive(False)
             self.toolbar.hide()
             return
+        else:
+            self.im_combo.set_sensitive(True)
 
         if not self.im_combo.get_model():
             print("no model")
@@ -770,7 +780,7 @@ class MintLocale:
         self.install_label.set_markup("<b>%s</b>\n<small>%s</small>" % (_("Language support"), gettext.ngettext("%d language installed", "%d languages installed", num_installed) % num_installed))
 
     def accountservice_ready(self, user, param):
-        self.builder.get_object("main_window").show()
+        self.window.show()
 
     def accountservice_changed(self, user):
         print("AccountsService language is: '%s'" % user.get_language())
@@ -817,7 +827,8 @@ class MintLocale:
                 all_locales_are_utf8 = False
                 break
 
-        for line in locales.split("\n"):
+        built_locales = {}
+        for line in locales.rstrip().split("\n"):
             line = line.replace("utf8", "UTF-8")
             cur_index += 1
             locale_code = line.split(".")[0].strip()
@@ -863,6 +874,10 @@ class MintLocale:
                 flag = flag_path
             else:
                 flag = FLAG_PATH % '_generic'
+
+            built_locales[language_label] = (line, flag)
+
+        for language_label, (line, flag) in sorted(built_locales.items()):
             locale = Locale(line, language_label)
             self.locale_button.add_picture(flag, self.set_user_locale, title=language_label, id=locale)
             self.region_button.add_picture(flag, self.set_user_region, title=language_label, id=locale)
